@@ -85,6 +85,39 @@
 ;; (greader-dict-mode 1)
 ;; (greader-dict--set-file 'mode)))
 ;;
+;; filters
+;; Filters are an alternative way of implementing your pronunciation
+;; rules.
+;; Word and match abstractions exist to make simpler daily tasks, or,
+;; put in other words, matches and words are regexp presets.
+;; Matches in particular have the limitation that you must define a
+;; rule in terms of human language entities, matches must have
+;; necessarily an alphabetic part.
+;; You cannot use matches to define rules for character substitution.
+;; Filters instead allow you to use whatever you want in terms of
+;; regexps, and to substitute them with all the constructs that the
+;; regexp matcher of emacs allow.
+;; filters however can become fastly inefficent, because the algoritm
+;; used is crude when applying filters: take every filter and cycle
+;; over the buffer until you have applied all the filters.
+;; the dictionary feature, instead, can handle thousands of
+;; definitions with a small decrease of performance.
+;; I suggest that you should use word definitions when possible, even
+;; if those are similar.
+;; Use matches when you individuate a pattern that can work with a
+;; common set of characters, using `shy groups'.
+;; See the emacs manual for more information about regexp syntax and
+;; related.
+;; Use `greader-dict-filter-add' to add a new filter to the database,
+;; `greader-dict-filter-remove' to remove an existing filter from the
+;; database,
+;; `greader-dict-filter-modify' to modify a key preserving its old
+;; value.
+;; to enable filters, use the command `greader-dict-toggle-filters',
+;; it is also a customizable variable.
+;; This module offers a command
+;; `greader-dict-pronounce-in-other-language' that can be used for
+;; earing how the tts pronounces a word in another language.
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; 
 ;;; Change Log:
@@ -110,6 +143,17 @@
 ;; 
 ;;; Code:
 (require 'greader)
+(defgroup greader-dict nil
+  "String substitution module for greader"
+  :group 'greader)
+;; THanks to the loved and alwais useful elisp reference.
+(defun string-hash-ignore-case (a)
+  (sxhash-equal (upcase a)))
+
+(define-hash-table-test 'ignore-case
+			'string-equal-ignore-case
+			'string-hash-ignore-case)
+
 (defvar-local  greader-dictionary nil)
 (defvar greader-dict-match-indicator "\%\*"
   "Regexp that will be used for match delimiter.")
@@ -139,9 +183,10 @@
 
 (defun greader-dict-substitute-word (match)
   "Substitute MATCH only if it constitutes an entire word."
+
   (save-excursion
     (goto-char (point-min))
-    (let ((word (concat "\\(\\W+\\)" "\\(" match "\\)" "\\(\\W+\\)"))
+    (let ((word (concat "\\(\\W?\\)" "\\(" match "\\)" "\\(\\W+\\)"))
 	  (alternative-word
 	   (concat "\\(^\\)" "\\(" match "\\)" "\\(\\W+\\)"))
 	  (end-word
@@ -150,7 +195,6 @@
 						 alternative-word nil
 						 t)
 		 (re-search-forward end-word nil t))
-	;; (edebug)
 	(let ((replacement
 	       (concat (match-string 1)
 		       (gethash match greader-dictionary)
@@ -169,7 +213,11 @@ A value of 0 indicates saving immediately."
   "Add the WORD REPLACEMent pair to `greader-dictionary'.
 If you want to add a partial replacement, you should
 add `\*'to the end of the WORD string parameter."
-  (puthash (downcase word) replacement greader-dictionary)
+  ;; We prevent an infinite loop if disallowing that key and values
+  ;; are the same.
+  (when (string-equal-ignore-case word replacement)
+    (user-error "key and value are the same, aborting"))
+  (puthash word replacement greader-dictionary)
   (setq greader-dict--saved-flag nil)
   (cond
    ((> greader-dict-save-after-time 0)
@@ -202,34 +250,67 @@ add `\*'to the end of the WORD string parameter."
    (t
     (setq greader-dict--saved-flag t)
     nil)))
+(defvar greader-dict--item-type-alist '((match
+					 . "\%\*")
+					(filter
+					 . "\%f")
+					(word . ""))
+  "item types and relative prefixes.")
 
 (defun greader-dict-item-type (key)
   "Return the type of KEY.
 Possible return values are:
 `word' for a wole word,
-`match' for partial matches.
+`match' for partial matches,
+`filter' for filters.
 There may be more in the future.
 Return nil if KEY is not present in `greader-dictionary'."
-  (cond
-   ((not key)
-    nil)
-   ((not (string-suffix-p greader-dict-match-indicator key))
-    'word)
-   ((string-suffix-p greader-dict-match-indicator key)
-    'match)
-   (t nil)))
+  (when key
+    (let (result)
+      (catch 'key-found
+	(dolist (type greader-dict--item-type-alist)
+	  (if
+	      (gethash
+	       (concat key
+		       (unless (string-suffix-p (cdr type) key)
+			 (cdr type)))
+	       greader-dictionary)
+	      (progn
+		(setq result (car type))
+		(throw 'key-found result))
+	    nil))))))
 
 (defun greader-dict--get-key-from-word (word)
   "Return key related to WORD, nil otherwise."
   (setq word (string-trim word))
-  (let ((key nil))
-    (maphash (lambda (k v)
-	       (when (string-match (string-remove-suffix
-				    greader-dict-match-indicator k)
-				   word)
-		 (setq key k)))
-	     greader-dictionary)
-    key))
+  (cond
+   ((gethash word greader-dictionary)
+    word)
+   (t
+    (let ((reduced-dictionary (make-hash-table :test 'ignore-case)))
+      (dolist (item (greader-dict--get-matches 'match))
+	(puthash item (gethash item greader-dictionary)
+		 reduced-dictionary))
+      (let ((key nil))
+	(catch 'matched
+	  (maphash
+	   (lambda (k _v)
+	     (let* ((result (string-remove-suffix
+			     greader-dict-match-indicator k))
+		    (candidate-matches (string-split result "\\W" t)))
+	       (setq candidate-matches (sort candidate-matches
+					     (lambda (s1 s2) (>
+							      (length
+							       s1)
+							      (length
+							       s2)))))
+	       (dolist (candidate candidate-matches)
+		 ;; (message "%s" (concat "matching " candidate " against " word "..."))
+		 (when (string-match candidate word)
+		   ;; (message "Matched!")
+		   (setq key (concat k greader-dict-match-indicator))
+		   (throw 'matched key)))))
+	   reduced-dictionary)))))))
 
 ;; This function checks that, in the string you pass to it, there are
 ;; effectively words to be replaced. If so, use apis
@@ -245,26 +326,38 @@ Return nil if KEY is not present in `greader-dictionary'."
 						 greader-dict--current-reading-buffer))
     (insert text)
     (goto-char (point-min))
-    (let ((inhibit-read-only t))
-      (re-search-forward "\\w" nil t)
-      (while (not (eobp))
-	(let*
-	    ((key (greader-dict--get-key-from-word (downcase
-						    (thing-at-point
-						     'word))))
-	     (modified-word
-	      (concat (downcase (thing-at-point 'word))
-		      greader-dict-match-indicator)))
-	  (cond
-	   ((equal (greader-dict-item-type key) 'word)
-	    (greader-dict-substitute-word (string-remove-suffix
-					   greader-dict-match-indicator
-					   key)))
-	   ((equal (greader-dict-item-type key) 'match)
-	    (greader-dict-substitute-match key))
-	   ((not (greader-dict-item-type key))
-	    nil)))
-	(re-search-forward "\\W*\\w" nil 1))
+    (when
+	(buffer-local-value 'greader-dict-toggle-filters
+			    greader-dict--current-reading-buffer) 
+      (setq greader-filters (buffer-local-value 'greader-filters
+						greader-dict--current-reading-buffer))
+      (greader-dict-filters-apply))
+    (if
+	(buffer-local-value 'greader-dict-mode
+			    greader-dict--current-reading-buffer)
+	(progn
+	  ;; We check if text is actually just one word, and in that case
+	  ;; insert a new line at end of temp buffer.
+	  (when (= (count-words (point-min) (point-max)) 1)
+	    (save-excursion (goto-char (point-max)) (newline)))
+	  (let ((inhibit-read-only t))
+	    (re-search-forward "\\w" nil t)
+	    (while (not (eobp))
+	      (let*
+		  ((key (greader-dict--get-key-from-word (
+							  thing-at-point
+							  'word))))
+		(cond
+		 ((equal (greader-dict-item-type key) 'word)
+		  (greader-dict-substitute-word (string-remove-suffix
+						 greader-dict-match-indicator
+						 key)))
+		 ((equal (greader-dict-item-type key) 'match)
+		  (greader-dict-substitute-match key))
+		 ((not (greader-dict-item-type key))
+		  nil)))
+	      (re-search-forward "\\W*\\w" nil 1))
+	    (buffer-string)))
       (buffer-string))))
 
 ;; This function saves the contents of the hash table.
@@ -290,7 +383,7 @@ Return nil if KEY is not present in `greader-dictionary'."
 				 greader-dict--current-reading-buffer))
     (maphash
      (lambda (k v)
-       (insert (concat k "=" v "\n")))
+       (insert (concat "\"" k "\"" "=" v "\n")))
      greader-dictionary)
     (write-region (point-min) (point-max)
 		  (greader-dict--get-file-name)))
@@ -300,9 +393,9 @@ Return nil if KEY is not present in `greader-dictionary'."
   "populate `greader-dictionary' with the contents of
 `greader-dict-filename'.
 If FORCE is non-nil, reading happens even if there are definitions not
-  yet saved.
+yet saved.
 If FORCE is nil \(the default\) then this function generates an
-  user-error and aborts the reading process."
+user-error and aborts the reading process."
   ;; This code is to provide a variable
   ;; `greader-dictionary' by default usable in the buffer
   ;; temporary where the replacements defined in `greader-after-get-sentence-functions' occur.
@@ -318,11 +411,13 @@ If FORCE is nil \(the default\) then this function generates an
       (insert-file-contents (greader-dict--get-file-name))
       (when-let ((lines (string-lines (buffer-string) t)))
 	(dolist (line lines)
-	  (setq line (split-string line "="))
+	  (setq line (split-string line "=" ))
+	  (setf (car line) (car (split-string (car line) "\"" t)))
 	  (let ((greader-dict-save-after-time -1))
 	    (greader-dict-add (car line) (car (cdr line)))))
-	(setq greader-dict--saved-flag t))))
-  (add-hook 'buffer-list-update-hook #'greader-dict--update))
+	(greader-dict--filter-init)
+	(setq greader-dict--saved-flag t)))
+    (add-hook 'buffer-list-update-hook #'greader-dict--update)))
 
 ;; Command for saving interactively dictionary data.
 (defun greader-dict-save ()
@@ -332,7 +427,8 @@ You should use this command when you want to save your dictionary and
 Otherwise, data saving is done automatically when you add a definition
 to the dictionary."
   (interactive)
-  (greader-dict-write-file))
+  (let ((greader-dict--saved-flag nil))
+    (greader-dict-write-file)))
 
 ;; This command Adds a definition to `greader-dictionary'.
 ;; If the region is active and it does not constitute more than one word,
@@ -363,7 +459,11 @@ for definition and substitution, without defaults.
 If called with prefix argument, ask for a match.
 In this case you can type a regular expression.
 You can use regular expressions to, for example, craft filters instead
-of pronunciation rules."
+of pronunciation rules.
+If the customizable variable
+`greader-dict-include-sentences-in-defaults' is enabled, when adding
+an entry the defaults will include also the set of words that makes up
+the current sentence."
   (interactive "P")
   (let (key value)
     (cond
@@ -374,7 +474,8 @@ of pronunciation rules."
 	(user-error "Input is empty: aborting"))
       (setq key (concat key greader-dict-match-indicator))
       (setq value (read-string (concat "substitute regexp " key "with:
-")))
+")
+			       nil nil(gethash key greader-dictionary)))
       (greader-dict-add key value))
      ((region-active-p)
       (when (= (count-words(region-beginning) (region-end)) 1)
@@ -385,21 +486,32 @@ of pronunciation rules."
 					(region-end)))
 			  greader-dict-match-indicator))
 	(setq value (read-string (concat "substitute match " key
-					 "with:")))
+					 "with: ")
+				 nil nil
+				 (gethash key greader-dictionary)))
 	(greader-dict-add key value)))
      ((not (region-active-p))
       (if-let ((default-word (thing-at-point 'word)))
 	  (progn
 	    (setq key (read-string "Original word to substitute or
-modify: " nil
-nil
-(append (list default-word)(greader-dict--get-matches 'word))))
+modify: "
+				   nil
+				   nil
+				   (append (list default-word)(when
+								  greader-dict-include-sentences-in-defaults
+								(greader-dict--get-word-alternatives
+								 (greader-get-sentence)))
+					   (greader-dict--get-matches
+					    'word))))
 	    (setq value (read-string (concat "substitute word " key
-					     " with:")))
+					     " with: ")
+				     (gethash key greader-dictionary)))
 	    (greader-dict-add key value))
 	(setq key (read-string "Word to add or modify: " nil nil
 			       (greader-dict--get-matches 'word)))
-	(setq value (read-string (concat "substitute " key " with:")))
+	(setq value (read-string (concat "substitute " key " with: ")
+				 nil nil
+				 (gethash key greader-dictionary)))
 	(greader-dict-add key value))))))
 
 (defun greader-dict-remove-entry (key)
@@ -430,12 +542,13 @@ Please use `greader-dict-save' for that purpose."
   "C-r d a" #'greader-dict-add-entry
   "C-r d k" #'greader-dict-remove-entry
   "C-r d c" #'greader-dict-change-dictionary
+  "C-r d l" #'greader-dict-pronounce-in-other-language
   "C-r d s" #'greader-dict-save)
 (defun greader-dict--replace-wrapper (text)
   "Function to add to `greader-after-get-sentence-functions'.
 It simply calls `greader-dict-check-and-replace' with TEXT as its
 argument, only if `greader-dict-mode' is enabled."
-  (if greader-dict-mode
+  (if (or greader-dict-mode greader-dict-toggle-filters)
       (greader-dict-check-and-replace text)
     text))
 
@@ -548,6 +661,13 @@ asked."
 ;; (remove-hook 'buffer-list-update-hook #'greader-dict--update)))))
 
 (defun greader-dict--update ()
+  (when greader-dict-toggle-filters
+    (setq greader-dict--current-reading-buffer (current-buffer))
+    (let ((dict-mode-state greader-dict-mode))
+      (greader-dict-mode 1)
+      (greader-dict-read-from-dict-file t)
+      (unless dict-mode-state
+	(greader-dict-mode -1))))
   (when greader-dict-mode
     (setq greader-dict--current-reading-buffer (current-buffer))
     (unless greader-dict--saved-flag
@@ -581,14 +701,16 @@ as a word definition."
   :lighter " gr-dictionary"
   (cond
    (greader-dict-mode
-    (setq greader-dictionary (make-hash-table :test 'equal))
+    (setq greader-dictionary (make-hash-table :test 'ignore-case))
     (setq greader-dict--current-reading-buffer (current-buffer))
     (greader-dict-read-from-dict-file)
     (add-hook 'greader-after-get-sentence-functions
 	      #'greader-dict--replace-wrapper 1)
-					; (add-hook 'greader-reading-mode-hook #'greader-dict--update))))
-
-    (add-hook 'buffer-list-update-hook #'greader-dict--update))))
+    (add-hook 'buffer-list-update-hook #'greader-dict--update)
+    (add-hook 'greader-after-change-language-hook
+	      (lambda ()
+		(when greader-dict-mode
+		  (greader-dict-read-from-dict-file)))))))
 ;; Questa funzione è solo di utilità e potrebbe essere rimossa o
 ;; modificata in qualsiasi momento.
 (defun greader-dict-beep ()
@@ -598,10 +720,12 @@ as a word definition."
   "Print some information about current dictionary."
   (interactive)
   (let ((message
-	 (concat "Current dictionary is " (symbol-name (greader-dict--file-type))
+	 (concat "Current dictionary is "
+		 (symbol-name (greader-dict--file-type))
 		 " in file " greader-dict-filename " it has "
 		 (number-to-string (hash-table-count
-				    greader-dictionary)) " entries.")))
+				    greader-dictionary))
+		 " entries.")))
     (message "%s" message)))
 
 (defun greader-dict--get-matches (type &optional decorate)
@@ -609,24 +733,237 @@ as a word definition."
 If TYPE is `all', all items in the current dictionary will be included."
   (let ((matches nil))
     (maphash
-     (lambda (k v)
-       (cond ((equal (greader-dict-item-type k) type)
-	      (let ((match (string-remove-suffix
-			    greader-dict-match-indicator k)))
-		(when decorate
-		  (setq match (concat match " \(" (gethash k greader-dictionary) "\)")))
-		(push match matches)))
-	     ((equal type 'all)
-	      (let ((match (string-remove-suffix
-			    greader-dict-match-indicator k)))
-		(when decorate
-		  (setq match (concat match " \(" (gethash k greader-dictionary) "\)")))
-		(push match matches))))) greader-dictionary)
+     (lambda (k _v)
+       (cond
+	((equal (greader-dict-item-type k) type)
+	 (let ((match (and (string-remove-suffix
+			    greader-dict-match-indicator k)
+			   (string-remove-suffix
+			    greader-dict-filter-indicator
+			    k))))
+	   (when decorate
+	     (setq match
+		   (concat match " \(" (gethash k greader-dictionary)
+			   "\)")))
+	   (push match matches)))
+	((equal type 'all)
+	 (let
+	     ((match
+	       (string-remove-suffix greader-dict-match-indicator k)))
+	   (when decorate
+	     (setq match
+		   (concat match " \(" (gethash k greader-dictionary)
+			   "\)")))
+	   (push match matches)))))
+     greader-dictionary)
     (sort
      matches
      (lambda (s1 s2)
        (string-greaterp s2 s1)))))
+(defun greader-dict-modify-key (arg)
+  "Modify a key (_NOT_ a value associated with it!).
+While `greader-dict-add-entry can modify either keys or associated
+values, `greader-dict-modify-key' allows you to modify the key
+itself, without modifying the associated value.
+if prefix ARG is non-nil, then this command proposes only keys that
+are classified as matches."
+  (interactive "P")
+  (let ((key (read-string "key to modify: " nil nil (if arg
+							(greader-dict--get-matches
+							 'match)
+						      (greader-dict--get-matches
+						       'all))))
+	(new-key nil))
+    (unless key
+      (user-error "Key not valid"))
+    (if-let ((backup-value (gethash key greader-dictionary)))
+	(progn
+	  (setq new-key (read-string (concat "substitute key " key "
+  with:")
+				     nil nil key))
+	  (unless new-key
+	    (user-error "Invalid replacement"))
+	  (greader-dict-remove key)
+	  (greader-dict-add new-key backup-value))
+      (user-error "Key not found"))))
 
+(defcustom greader-dict-include-sentences-in-defaults nil
+  "Includi le parole della frase come alternative.
+When active, the constituent words of the sentence currently in
+reading will be added to the list of defaults (where it makes sense
+to do it).
+In this way anyone who wishes can search for the word to manipulate
+using a menu instead of navigating the buffer."
+  :type 'boolean)
+
+(defun greader-dict--get-word-alternatives (text)
+  "Return a list with a set of words in TEXT."
+  (if-let ((alternatives text))
+      (progn
+	(setq alternatives nil)
+	(dolist (word (split-string text "\\W" t))
+	  (unless (member word alternatives)
+	    (push word alternatives)))
+	(reverse alternatives))
+    (user-error "No text")))
+
+(defvar greader-dict-lang-history nil)
+
+;;;###autoload
+(defun greader-dict-pronounce-in-other-language (word new-lang)
+  "pronounce WORD in the language specified by NEW-LANG.
+The currently configured backend will be used for the voice.
+NEW-LANG should be an ISO code, compatible with the back-end you are
+using.
+If the variable `greader-dict-include-sentences-in-defaults is enabled, when
+asked about the word to pronounce, the defaults will be a set of words
+in the current sentence."
+
+  (interactive
+   (list
+    (read-string "Word to pronounce: " nil t
+		 (when greader-dict-include-sentences-in-defaults
+		   (greader-dict--get-word-alternatives
+		    (greader-get-sentence))))
+    (read-string (concat "language in which you wish to listen:") nil
+		 'greader-dict-lang-history)))
+  (unless new-lang (user-error "No language specified"))
+  (let ((old-lang (greader-get-language)))
+    (greader-set-language new-lang)
+    (greader-read-asynchronous word)
+    (greader-set-language old-lang)))
+
+;; filters.
+;; filters allow users to define arbitrary regexps to be replaced
+;; either with empty strings or by another string.
+;; It is necessary to conceptually abstract filters from other types of
+;; match because the filters allow the use of any character and in any case, being applied as they are,
+;; the user can better exploit the expressive power of regexps.
+;; so filters are a separate feature, which we can consider an
+;; "advanced" use case of greader-dict.
+
+(defvar-local greader-filters nil
+  "Hash table containing our filters.")
+
+(defvar greader-dict-filter-indicator "\%f")
+
+(defvar-keymap greader-dict-filter-map
+  :doc "key bindings for greader-dict filter feature."
+  "C-r d f a" #'greader-dict-filter-add
+  "C-r d f m" #'greader-dict-filter-modify
+  "C-r r" #'isearch-backward
+  "C-r d f k" #'greader-dict-filter-remove)
+
+;;;###autoload
+(define-minor-mode greader-dict-toggle-filters
+  "enable or disable filters.
+Filters allow you to replace every regexp you wish with something
+else you wish.
+While matches and words are conceived as facilities that are
+designated to be user-friendly interfaces to regexps, with filters
+you can unleash all
+your expressiveness!
+Filters and dictionary are considered independent features for now, so
+you can enable filters without the extra payload given by
+`greader-dict-mode'.
+To use a filter you must first enable this mode, and, eventually, add
+a filter.
+So use `greader-dict-filter-add' to do that.
+When you are prompted for the filter, you should insert the regexp
+that must match to have the associated replacement.
+You can use the usual `\\\\' expressions, shy groups and all the power
+of regexps.
+If you are interested in how to write a regexp please consult the info
+node `(emacs) Regexps'."
+  :keymap greader-dict-filter-map
+  :lighter " gr-filters"
+  (when greader-dict-toggle-filters
+    (setq greader-filters (make-hash-table :test 'ignore-case))
+    (setq greader-dict--current-reading-buffer (current-buffer))
+    (unless greader-dictionary
+      (greader-dict-mode 1)
+      (greader-dict-mode -1))
+    (greader-dict--filter-init)))
+
+(defun greader-dict--is-filter-p (key)
+  "Return t if KEY is a filter based on
+`greader-dict-filter-indicator'."
+  (string-suffix-p greader-dict-filter-indicator key))
+
+(defun greader-dict-filter-add (key value)
+  "Add KEY as a filter with associated VALUE."
+  (interactive
+   (let*
+       ((key (read-string "filter (regexp) to add or modify: " nil nil
+			  (greader-dict--get-matches 'filter)))
+	(value (read-string
+		(concat "substitute regexp " key " with: ") nil nil
+		(gethash (concat key greader-dict-filter-indicator)
+			 greader-dictionary))))
+     (list key value)))
+  (greader-dict-add (concat key greader-dict-filter-indicator) value)
+  (greader-dict--filter-init))
+
+(defun greader-dict-filter-remove (key)
+  "Remove KEY from the hash-table."
+  (interactive
+   (let ((key (read-string "filter to remove: " nil nil
+			   (greader-dict--get-matches 'filter))))
+     (list key)))
+  (when (gethash (concat key greader-dict-filter-indicator)
+		 greader-dictionary)
+    (greader-dict-remove (concat key
+				 greader-dict-filter-indicator))
+    (greader-dict--filter-init)))
+
+(defun greader-dict-filter-modify (key new-key)
+  "Modify _Only_ KEY.
+KEY must be an existing filter key, this command provides a list of
+them accessible using the history commands.
+NEW-KEY must not be empty, if empty, this command will signal an error."
+
+  (interactive
+   (let* ((key (read-string "key to modify: " nil nil
+			    (greader-dict--get-matches 'filter)))
+	  (new-key (read-string "new value: " nil nil key)))
+     (unless
+	 (gethash (concat key greader-dict-filter-indicator)
+		  greader-dictionary)
+       (user-error "Invalid key"))
+     (when (string-empty-p new-key)
+       (user-error "Invalid replacement, string is empty"))
+     (list key new-key)))
+  (let ((old-value (gethash (concat key greader-dict-filter-indicator)
+			    greader-dictionary)))
+    (greader-dict-filter-remove key)
+    (greader-dict-filter-add new-key old-value))
+  (greader-dict--filter-init))
+
+(defun greader-dict--filter-init ()
+  "Initialize filters hash table.
+It works by subtracting from `greader-dictionary' the entries that are
+classified as filters and, eventually, adding them to the filters
+hash table."
+  (when greader-filters (clrhash greader-filters))
+  (maphash
+   (lambda (k v)
+     (when (and greader-dict-toggle-filters (string-suffix-p
+					     greader-dict-filter-indicator
+					     k))
+       (puthash k v greader-filters)))
+   greader-dictionary))
+
+(defun greader-dict-filters-apply ()
+  "Apply filters defined in sequence to the current buffer."
+  (maphash
+   (lambda (k v)
+     (save-excursion
+       (goto-char (point-min))
+       (while (re-search-forward (string-remove-suffix
+				  greader-dict-filter-indicator k)
+				 nil t)
+	 (replace-match v))))
+   greader-filters))
 
 (provide 'greader-dict)
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
