@@ -6,7 +6,7 @@
 ;; Author: Michelangelo Rodriguez <michelangelo.rodriguez@gmail.com>
 ;; Keywords: tools, accessibility
 
-;; Version: 0.9.0
+;; Version: 0.9.20
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -561,7 +561,7 @@ mindfullness!)."
 (defun greader-read (&optional goto-marker)
   "Start reading of current buffer.
 if `GOTO-MARKER' is t and if you pass a prefix to this
-  function, point jumps at the last position you called command `greader-read'."
+function, point jumps at the last position you called command `greader-read'."
 
   (interactive "P")
   (when goto-marker
@@ -622,13 +622,24 @@ Argument ARG is not used."
     (get-buffer-create greader-debug-buffer)
     (set-buffer greader-debug-buffer)
     (insert arg)))
+(defvar greader-sentence-regexp "[.?!]+[[:space:]]"
+  "Regexp to indicate the end of a sentence in terms of greader.")
 
 (defun greader-forward-sentence ()
   "Move the point to next sentence."
   (let ((result (greader-call-backend 'next-text)))
     (if (not (equal result 'not-implemented))
 	result
-      (forward-sentence))))
+      (let ((sentence-end (if greader-classic-nav-mode nil greader-sentence-regexp)))
+	(forward-sentence)))))
+
+(defun greader-backward-sentence ()
+  "Move back by a sentence in terms of greader."
+  (let ((result (greader-call-backend 'next-text)))
+    (if (not (equal result 'not-implemented))
+	result
+      (let ((sentence-end (if greader-classic-nav-mode nil greader-sentence-regexp)))
+	(backward-sentence)))))
 
 (defun greader-get-sentence ()
   "Get current sentence.
@@ -642,7 +653,7 @@ If at end of buffer, nil is returned."
 	(setq sentence-start (point))
 	(save-excursion
 	  (when (not (eobp))
-	    (forward-sentence))
+	    (greader-forward-sentence))
 	  (if (> (point) sentence-start)
 	      (string-trim (buffer-substring sentence-start (point)) "[ \t\n\r]+")
 	    nil))))))
@@ -657,11 +668,13 @@ Optional argument STRING contains the string passed to
 `greader-read-asynchronous'."
   (if greader-filter-enabled
       (message string)))
-
+(defvar greader-after-change-language-hook nil
+  "The functions stored in this variable are executed just after new
+language is set.")
 (defun greader-set-language (lang)
   "Set language of tts.
 LANG must be in ISO code, for example `en' for English or `fr' for
-French.  This function sets the language of tts local for current
+French.  This function sets the language of tts locally for current
 buffer, so if you want to set it globally, please use
 `M-x customize-option RET greader-language RET'."
   (interactive
@@ -670,7 +683,9 @@ buffer, so if you want to set it globally, please use
       (if (equal result 'not-implemented)
 	  (read-string "Set language to: ")
 	result))))
-  (greader-call-backend 'lang lang))
+  (greader-call-backend 'lang lang)
+  (run-hooks 'greader-after-change-language-hook))
+
 (defun greader-set-punctuation (flag)
   "Set punctuation to FLAG."
   (greader-call-backend 'punctuation flag))
@@ -699,17 +714,9 @@ get the language from the environment."
 (defun greader-toggle-punctuation ()
   "Toggle punctuation locally for current buffer."
   (interactive)
-  (if (not (greader-call-backend 'punctuation))
-      (progn
-	(greader-tts-stop)
-	(greader-set-punctuation 'yes)
-	(message "punctuation enabled in current buffer")
-	(greader-read))
-    (progn
-      (greader-tts-stop)
-      (greader-set-punctuation 'no)
-      (message "punctuation disabled in current buffer")
-      (greader-read))))
+  (greader-stop)
+  (greader-call-backend 'punctuation 'toggle)
+  (greader-read))
 
 (defun greader-toggle-timer-flag ()
   "Not yet documented."
@@ -1202,10 +1209,14 @@ When called from a function, you should specify SRC and DST, even if
 
 (defvar greader--timer-backward nil)
 
+(defcustom greader-classic-nav-mode nil
+  "Enable classic navigation while reading."
+  :type 'boolean)
+
 (defun greader--forward ()
-  (when (equal
-	 (point) greader--marker-backward)
-    (forward-sentence)
+  (when (and (equal
+	 (point) greader--marker-backward) greader-reading-mode)
+    (greader-forward-sentence)
     (backward-char 2)
     (when greader-backward-acoustic-feedback
       (beep))))
@@ -1216,11 +1227,11 @@ When called from a function, you should specify SRC and DST, even if
 (defun greader-backward ()
   "Restart reading from start of sentence.
 When at start of sentence, this function sets a timer for
-  `greader--set-forward-timer' seconds, and, if the cursor is yet at
-  start of sentence, puts the point at the end.
+`greader--set-forward-timer' seconds, and, if the cursor is yet at
+start of sentence, puts the point at the end.
 So you can use this command like a player, if you press <left> you
-  will ear the start of last read sentence, and if you press again
-  while timer is in effect, you go back by one sentence."
+will ear the start of last read sentence, and if you press again
+while timer is in effect, you go back by one sentence."
   (interactive)
   (when (bobp)
     (signal 'beginning-of-buffer ()))
@@ -1228,7 +1239,7 @@ So you can use this command like a player, if you press <left> you
     (cancel-timer greader--timer-backward)
     (setq greader--timer-backward nil))
   (greader-tts-stop)
-  (backward-sentence)
+  (greader-backward-sentence)
   (greader-set-register)
   (setq greader--marker-backward (point))
   (greader--set-forward-timer)
@@ -1250,7 +1261,7 @@ So you can use this command like a player, if you press <left> you
 
 (defvar-local greader-queue nil
   "This is the variable that contains the queue.
-each element is formed by a cons, whose form is 'start-position
+each element is formed by a cons, whose form is `start-position
 . end-position'.")
 
 (defvar-local greader-queue-current-element nil
@@ -1372,7 +1383,7 @@ If the current element is the first, it returns it."
 
 ;; This function will act as a sentinel for the tts when invoked
 ;; by the `greader-queue-read' function.
-(defun greader-queue-next-action (&optional process string)
+(defun greader-queue-next-action (&optional _process _string)
   "Sentinel process for greader-queue-mode."
   (setq greader-queue-current-element (1+
 				       greader-queue-current-element))
@@ -1444,10 +1455,12 @@ before.")
 
 ;;;###autoload
 (define-minor-mode greader-queue-mode
-  "In this mode, text reading occurs via blocks.
-normally greader reads the text of a buffer sequentially; in queue-mode you add blocks of text to a queue, so you can choose only certain parts of the buffer.
-As reading progresses, further blocks can be added,
-or you can add the blocks and then start reading."
+  "Read buffers as blocks of text.
+In this mode, text reading occurs via blocks.  normally greader
+reads the text of a buffer sequentially; in queue-mode you add blocks
+of text to a queue, so you can choose only certain parts of the
+buffer.  As reading progresses, further blocks can be added, or you
+can add the blocks and then start reading."
   :lighter " greader-q"
   (cond
    (greader-queue-mode
@@ -1469,12 +1482,12 @@ If it is a function, it must return a string."
   "Return the value of `greader-enriched-tag'."
   (pcase greader-enriched-tag
     ((pred functionp)
-     (funcall #'greader-enriched-tag))
+     (with-no-warnings (funcall #'greader-enriched-tag)))
     ((pred stringp)
      greader-enriched-tag)))
 
 (defun greader-scrap-links (input-string)
-  "Modify the INPUT-STRING string to precede each link with 'link: '"
+  "Modify the INPUT-STRING string to precede each link with `link: '"
   (let ((pos 0)
         (modified nil)
         (result (copy-sequence input-string)))
@@ -1537,8 +1550,10 @@ guessing."
   :type 'string)
 
 (defun greader-continuous-guess-function ()
-  "Guess the function for greader-continuous mode based on GREADER-CONTINUOUS-KEY.
-If GREADER-CONTINUOUS-KEY is nil, checks against `greader-continuous-excluded-modes'
+  "Guess the function for greader-continuous mode.
+this guessing is based on GREADER-CONTINUOUS-KEY.
+If GREADER-CONTINUOUS-KEY is nil, checks against
+`greader-continuous-excluded-modes'
 and `greader-continuous-modes' to determine the appropriate function."
   (cond
    ((member major-mode greader-continuous-excluded-modes)
